@@ -38,7 +38,7 @@ string strerror (int _code, const string & _message)
                             buffer.data(), (DWORD)buffer.size() - 1, NULL);
 
   std::stringstream message;
-  message << _message << ": " 
+  message << _message << ": "
           << ((ret > 0) ? buffer.data() : "system error message could not be fetched");
 
   return message.str().substr(0, message.str().find_last_not_of("\r\n\t"));
@@ -147,7 +147,7 @@ struct StartupInfo {
 
     pipe_holder in(sa), out(sa), err(sa);
 
-    // Ensure the write handle to the pipe for STDIN is not inherited. 
+    // Ensure the write handle to the pipe for STDIN is not inherited.
     if (!::SetHandleInformation(in.write, HANDLE_FLAG_INHERIT, 0)) {
       throw subprocess_exception(GetLastError(), "could not set handle information");
     }
@@ -186,7 +186,7 @@ struct StartupInfo {
      * still don't understand how to do it propertly, and there seems
      * to be a great deal of confusion among people online how to do
      * it correctly).
-     * 
+     *
      * _creation_flags |= CREATE_NEW_CONSOLE;
      */
   }
@@ -219,14 +219,14 @@ struct StartupInfo {
                         FILE_ATTRIBUTE_NORMAL,  // normal file
                         NULL);                  // no attr. template
 
-    if (output == INVALID_HANDLE_VALUE) { 
+    if (output == INVALID_HANDLE_VALUE) {
       return -1;
     }
 
     DuplicateHandle(output, &_process->pipe_stdout);
     DuplicateHandle(output, &_process->pipe_stderr);
     DuplicateHandle(output, error.address());
-  
+
     _si->cb = sizeof(STARTUPINFO);
     _si->hStdError = error;
     _si->hStdOutput = output;
@@ -240,7 +240,7 @@ struct StartupInfo {
    * The actual startup info object.
    */
   STARTUPINFO info;
- 
+
 };
 
 
@@ -251,7 +251,7 @@ static HANDLE CreateAndAssignChildToJob (HANDLE _process)
   if (!job_handle) {
     throw subprocess_exception(::GetLastError(), "group termination: could not create a new job");
   }
-  
+
   JOBOBJECT_EXTENDED_LIMIT_INFORMATION info;
   ::memset(&info, 0, sizeof(JOBOBJECT_EXTENDED_LIMIT_INFORMATION));
 
@@ -333,7 +333,7 @@ void process_handle_t::spawn (const char * _command, char *const _arguments[],
   /* if termination mode is "group" add process to the job; see here
    * for more details:
    * https://msdn.microsoft.com/en-us/library/windows/desktop/ms684161(v=vs.85).aspx
-   * 
+   *
    * "After a process is associated with a job, by default any child
    *  processes it creates using CreateProcess are also associated
    *  with the job."
@@ -403,14 +403,14 @@ size_t process_handle_t::read (pipe_type _pipe, int _timeout)
 {
   stdout_.clear();
   stderr_.clear();
-  
+
   ULONGLONG start = GetTickCount64();
   int timediff, sleep_time = 100; /* by default sleep 0.1 seconds */
-  
+
   if (_timeout >= 0) {
     sleep_time = _timeout / 10;
   }
-  
+
   do {
     size_t rc1 = 0, rc2 = 0;
     if (_pipe & PIPE_STDOUT) rc1 = stdout_.read(pipe_stdout);
@@ -420,11 +420,11 @@ size_t process_handle_t::read (pipe_type _pipe, int _timeout)
     if (rc1 > 0 || rc2 > 0 || sleep_time == 0) {
       return std::max(rc1, rc2);
     }
-    
+
     // sleep_time is now guaranteed to be positive
     Sleep(sleep_time);
     timediff = (int)(GetTickCount64() - start);
-    
+
   } while (_timeout < 0 || timediff < _timeout);
 
   // out of time
@@ -458,18 +458,18 @@ void process_handle_t::wait (int _timeout)
     _timeout = INFINITE;
 
   DWORD rc = ::WaitForSingleObject(child_handle, _timeout);
-  
+
   // if already exited
   if (rc == WAIT_OBJECT_0) {
     DWORD status;
     if (::GetExitCodeProcess(child_handle, &status) == FALSE) {
       throw subprocess_exception(::GetLastError(), "could not read child exit code");
     }
- 
+
     if (status == STILL_ACTIVE) {
       return;
     }
-    
+
     return_code = (int)status;
     state = EXITED;
   }
@@ -566,6 +566,21 @@ void process_handle_t::send_signal (int _signal)
 }
 
 
+bool process_exists (const pid_type & _pid) {
+  /*
+   * https://stackoverflow.com/questions/12900036/benefit-of-using-waitforsingleobject-when-checking-process-id
+   *
+   * When a process completes, it stops running but it doesn't go out of
+   * existence until the last handle to it is closed. The first solution
+   * distinguishes between those two states (still running or done running).
+   */
+  HANDLE h = OpenProcess(SYNCHRONIZE, FALSE, _pid);
+  DWORD ret = WaitForSingleObject(h, 0);
+  CloseHandle(h);
+  return (ret == WAIT_TIMEOUT);
+}
+
+
 /**
  * You have to Free() the buffer returned from this function
  * yourself - or let R do it, since we allocate it with Calloc().
@@ -634,15 +649,15 @@ int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int n
   const char * command = "x";
   char * args[] = { "x", NULL };
   char * env[]  = { NULL };
-  
+
   process_handle_t handle;
   if (spawn_process(&handle, command, args, env) < 0) {
     fprintf(stderr, "error in spawn_process\n");
     exit(EXIT_FAILURE);
   }
-  
+
   teardown_process(&handle);
-  
+
   return 0;
 }
 #endif /* WINDOWS_TEST */
