@@ -7,12 +7,16 @@ is_mac <- function ()
   identical(tolower(Sys.info()[["sysname"]]), 'darwin')
 }
 
+is_solaris <- function()
+{
+  identical(tolower(Sys.info()[["sysname"]]), 'sunos')
+}
 
 # --- R child ----------------------------------------------------------
 
 R_binary <- function ()
 {
-  binary <- ifelse(is_windows(), 'R.exe', 'R')
+  binary <- ifelse(is_windows(), 'Rterm.exe', 'R')
   binary <- file.path(R.home("bin"), binary)
   stopifnot(file.exists(binary))
   binary
@@ -28,24 +32,7 @@ R_child <- function(args = '--slave', ...)
 
 # --- OS interface -----------------------------------------------------
 
-process_exists <- function (handle)
-{
-  pid <- ifelse (is_process_handle(handle), as.integer(handle$c_handle), handle)
-  
-  if (is_windows()) {
-    output <- system2("tasklist", stdout = TRUE, stderr = TRUE)
-    return(length(grep(as.character(pid), output, fixed = TRUE)) > 0)
-  }
-  else {
-    flag <- ifelse(is_mac(), "-p", "--pid")
-    rc <- system2("ps", c(flag, as.character(pid)), stdout = NULL, 
-                  stderr = NULL)
-    return(rc == 0)
-  }
-}
-
-
-# wait_until_* 
+# wait_until_*
 #
 # Wait infinitey - on CRAN tests will timeout, locally we can always
 # tell that something is wrong. This is because some systems are simply
@@ -58,7 +45,7 @@ wait_until_appears <- function (handle)
     process_wait(handle, TIMEOUT_IMMEDIATE)
     if (process_state(handle) %in% c("exited", "terminated"))
       stop('failed to start ', handle$command, call. = FALSE)
-    Sys.sleep(.1)
+    Sys.sleep(.25)
   }
   return(TRUE)
 }
@@ -67,7 +54,7 @@ wait_until_appears <- function (handle)
 wait_until_exits <- function (handle)
 {
   while (process_exists(handle)) {
-    Sys.sleep(.1)
+    Sys.sleep(.25)
   }
   return(TRUE)
 }
@@ -75,6 +62,8 @@ wait_until_exits <- function (handle)
 
 terminate_gracefully <- function (handle, message = "q('no')\n")
 {
+  if (!process_exists(handle)) return(TRUE)
+
   if (!is.null(message)) {
     process_write(handle, message)
   }
